// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Twig;

namespace SafeHomeBase
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class SafeHomeBase : TwigGame
    {
        public SafeHomeBase() : base()
        { }

        protected override void InitializeTwigObjects()
        {
            Components.Add(new StupidCamera(this));
            Components.Add(new GroundPlane(this));
            InstallStandardDebuggingComponents();

#if Doll
            Doll doll= new Doll("doll", this);
            doll.Color = Color.WhiteSmoke;
            Components.Add(doll);
#endif
            //new Ball("ball2", this, new Vector3(-5, 0, -8), Color.Yellow);
            Adult parent = new Adult(this, "parent");
            Components.Add(parent);
            parent.PoseAt(new Vector3(1f, 0, 1.5f), Vector3.Normalize(new Vector3(1, 0, 1)));
            parent.Color = new Color(200, 0, 200);
            parent.UpdateThunks += delegate()
            {
                parent.legs.left.end.Velocity = Vector3.Zero;
                parent.legs.right.end.Velocity = Vector3.Zero;
            };

            Child kid = new Child(this, "little kid", 1, parent);
            Components.Add(kid);
            kid.generalCaretakerAvailability = kid.baseLineSecurity = 0.2f;
            Ball ball = new Ball(this, "ball", new Vector3(-5, 0, -5), Color.Yellow);
            kid.Attention.Focus = ball;
            Doll doll = new Doll(this, "doll");
            doll.Color = Color.White;
            Components.Add(doll);
            kid.arms.left.Grab(doll.spineTop, doll);
            kid.DeclareTypeAppraisal(typeof(Adult), new Mammal.ObjectAppraisal(0.2f, 1f));
            kid.DeclareTypeAppraisal(typeof(Ball), new Mammal.ObjectAppraisal(0.4f));
            kid.DeclareTypeAppraisal(typeof(MerryGoRound), new Mammal.ObjectAppraisal(0.4f));
            kid.DeclareTypeAppraisal(typeof(Child), new Mammal.ObjectAppraisal(-0.7f, 0.6f));
            kid.minimumGazeTime = 1f;


#if Doll
            kid.arms.left.Grab(doll.headTop, doll);
#endif
            Components.Add(ball);
            ball.UpdateThunks += delegate()
            {
                if (ball.Position.LengthSquared() > 400)
                {
                    ball.Nodes[0].Velocity = MathUtil.SetLength(ball.Nodes[0].Position, -10)-Physics.gVector;
                }
            };

            //kid.arms.left.end.trace = TraceMode.Acceleration;
            Child olderKid = new Child(this, "big kid", 1.2f, null);
            olderKid.generalCaretakerAvailability = 0.7f;
            kid.chartSection = 0;
            parent.chartSection = 1;
            //otherKid.chartSection = 1;
            parent.Attention.Focus = kid;
            olderKid.DeclareTypeAppraisal(typeof(Ball), new Mammal.ObjectAppraisal(0.4f));
            olderKid.DeclareTypeAppraisal(typeof(MerryGoRound), new Mammal.ObjectAppraisal(0.4f));
            olderKid.DeclareTypeAppraisal(typeof(Child), new Mammal.ObjectAppraisal(-0.8f));

            kid.LogMotion();
            olderKid.LogMotion();
            parent.LogMotion();

            Components.Add(olderKid);

            for (int i = 0; i < 5; i++)
            {
                Vector3 position = Vector3.Zero;
                while (position.LengthSquared() < 64)
                    position = MathUtil.NoiseVector(15);
                Components.Add(new Tree(this, position));
            }

            //kid.legs.left.end.Trace = TraceMode.Position | TraceMode.Update;

            MerryGoRound merryGoRound = new MerryGoRound(this, "merry-go-round", new Vector3(5f, 0, 3f));
            Components.Add(merryGoRound);
            olderKid.Attention.Focus = ball;
        }
    }
}
