// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Twig
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class ScriptTest : TwigGame
    {
        protected override void InitializeTwigObjects()
        {
            Components.Add(new Camera(this, "camera"));
            Components.Add(new GroundPlane(this));
            Components.Add(new TitleScreen(this, "titles"));
            InstallStandardDebuggingComponents();
            Components.Add(new ScriptScriptInterpreter(this, @"Script"));

            Child Michael = new Child(this, "Michael", 1, new Vector3(-1, 0, 6));
            Michael.chartSection = 0;
            Components.Add(Michael);
            Michael.maxSpeed /= 2f;
            Child Bryan = new Child(this, "Bryan", 1.2f, new Vector3(1, 0, 6));
            Components.Add(Bryan);
            Paper paper = new Paper(this, "checklist", "Clipboard", new Vector3(-1, 1, 7));
            Components.Add(paper);
            Pen pen = new Pen(this, "pencil", new Vector3(-1, 2, 7));
            Components.Add(pen);

            Paper script = new Paper(this, "script", "scriptprop", new Vector3(1, 1, 7));
            Components.Add(script);
            Bryan.Hold(script);

            Bryan.NewUtterance += delegate(TwigObject speaker, string speech)
            {
                if (speech == "Check!")
                    pen.Scribble(0.5f);
            };

            Components.Add(new BoxModel(this, "box", Content.Load<Model>("SixteenTonWeight"),
                            new Vector3(0, 0, 10),
                            2, 2, 2, false));

            Random rand = new Random();
            Marker searchMarker = new Marker(this, "searchMarker", Bryan.Position - Vector3.UnitX);
            Components.Add(searchMarker);
            searchMarker.UpdateThunks += delegate()
            {
                // If Bryan is looking at and also facing this marker, then move the marker randomly
                if (Bryan.LookingAt(searchMarker) && Bryan.FacingToward(searchMarker) && rand.Next(30)==0)
                    searchMarker.TeleportTo(Bryan.Position + new Vector3(2 * (float)rand.NextDouble() - 1, 0, 2 * (float)rand.NextDouble() - 1));
            };

            //Components.Add(new Paper(this, "paper", "Content\\checklist", new Vector3(0, 0, 9)));

        }
    }
}
