﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace TestClient
{
    public partial class ServerTestClient : Form
    {
        public ServerTestClient()
        {
            InitializeComponent();
            UpdateConnectButtons();
        }

        Socket server;
        NetworkStream output;

        public Socket Server
        {
            get {
                if (server == null)
                    ConnectToServer();
                return server;
            }
        }

        void ConnectToServer()
        {
            server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            server.Connect(hostnameTextBox.Text, Int32.Parse(portNumberTextBox.Text));
            DoReceive();
            output = new NetworkStream(server, FileAccess.Write);
            UpdateConnectButtons();
            outputWindow.Text = "Connecting to server...";
            firstPrint = true;
        }

        void DisconnectFromServer()
        {
            ReallyDisconnectFromServer();
            UpdateConnectButtons();
        }

        void ReallyDisconnectFromServer()
        {
            if (server != null && server.Connected)
            {
                try
                {
                    output.Flush();
                    server.Shutdown(SocketShutdown.Both);
                    server.Close();
                }
                catch
                {
                }
            }
            server = null;
        }

        void UpdateConnectButtons()
        {
            Thunk update = delegate()
            {
                if (server == null || !server.Connected)
                {
                    connectButton.Enabled = true;
                    disconnectButton.Enabled = false;
                    commandLineTextBox.Enabled = false;
                    sendButton.Enabled = false;
                    sendFileButton.Enabled = false;
                    AcceptButton = connectButton;
                    connectButton.Focus();
                    server = null;
                }
                else
                {
                    connectButton.Enabled = false;
                    disconnectButton.Enabled = true;
                    commandLineTextBox.Enabled = true;
                    sendButton.Enabled = true;
                    sendFileButton.Enabled = true;
                    AcceptButton = sendButton;
                    commandLineTextBox.Focus();
                }
            };
            if (connectButton.InvokeRequired)
                connectButton.Invoke(update);
            else
                update();
        }

        byte[] receiveBuffer = new byte[1024];
        delegate void Thunk();
        bool firstPrint = true;
        void DoReceive()
        {
            server.BeginReceive(receiveBuffer, 0, receiveBuffer.Length, SocketFlags.Partial,
                delegate(IAsyncResult ar)
                {
                    if (server == null || !server.Connected)
                        return;
                    try
                    {
                        int read = server.EndReceive(ar);
                        Thunk printer = delegate()
                        {
                            string text = new StreamReader(new MemoryStream(receiveBuffer, 0, read, false)).ReadToEnd();
                            if (firstPrint)
                            {
                                outputWindow.Text = text;
                                firstPrint = false;
                            }
                            else
                                outputWindow.AppendText(text);
                        };

                        outputWindow.Invoke(printer);
                        DoReceive();
                    }
                    catch
                    {
                        DisconnectFromServer();
                    }
                }, null);
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            ConnectToServer();
        }

        private void disconnectButton_Click(object sender, EventArgs e)
        {
            DisconnectFromServer();
        }

        protected override void OnClosed(EventArgs e)
        {
            ReallyDisconnectFromServer();
            base.OnClosed(e);
        }

        private void stressButton_Click(object sender, EventArgs e)
        {
            SendString("ls\r\n");
        }

        void SendString(string str)
        {
            byte[] bytes = System.Text.ASCIIEncoding.ASCII.GetBytes(str);
            output.Write(bytes, 0, bytes.Length);
#if old
            byte[] buffer = new byte[str.Length];
            MemoryStream s = new MemoryStream(buffer);
            StreamWriter w = new StreamWriter(s);
            w.Write(str);
            w.Close();
            s.Close();
            int sent = server.Send(buffer);
            if (sent != buffer.Length)
                MessageBox.Show(string.Format("Incomplete send.  Tried to send {0} bytes, only {1} transmitted.",
                    buffer.Length, sent));
#endif
        }

        private void sendButton_Click(object sender, EventArgs e)
        {
            string command = commandLineTextBox.Text+"\n";
            SendString(command);
            outputWindow.AppendText(command);
            commandLineTextBox.Text = null;
            commandLineTextBox.Focus();
        }

        private void sendFileButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog d = new OpenFileDialog();
            DialogResult r = d.ShowDialog();
            if (r == DialogResult.OK)
            {
                SendString(File.ReadAllText(d.FileName));
            }
            d.Dispose();
            commandLineTextBox.Focus();
        }
    }
}
