﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    /// <summary>
    /// A TwigObject whose model is a box and which acts as a supporting surface for other objects
    /// </summary>
    public class BoxModelSupportingSurface : BoxModel
    {
        /// <summary>
        /// Make a box-shaped object that acts as a supporting surface, given a model and position.
        /// </summary>
        public BoxModelSupportingSurface(TwigGame g, string name, Model m, Vector3 bottomCenter)
            : base(g, name, m, bottomCenter, true)
        {
        }

        /// <summary>
        /// Make a box-shaped object that acts as a supporting surface, given a model and position.
        /// If height is non-zero, it overrides the height of computed from the model.
        /// </summary>
        public BoxModelSupportingSurface(TwigGame g, string name, Model m, float height, Vector3 bottomCenter)
            : base(g, name, m, height, bottomCenter, true)
        {
        }

        /// <summary>
        /// SimpleTables are supporting surfaces
        /// </summary>
        public override bool IsASupportingSurface
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// The hight of the supporting surface is the height of the table
        /// </summary>
        public override float SupportingSurfaceLevel
        {
            get
            {
                return height;
            }
        }

        /// <summary>
        /// Always look at the top of the box
        /// </summary>
        public override Vector3 LookPoint
        {
            get
            {
                return TopPosition;
            }
        }

        /// <summary>
        /// Position is the bottom of the box
        /// </summary>
        public override Vector3 Position
        {
            get
            {
                return BottomPosition;
            }
        }

        /// <summary>
        /// True if the point is over the ground-plane projection of table.
        /// </summary>
        public override bool IsSupporting(Vector3 point)
        {
            Vector3 offset = point - Position;

            bool result = Math.Abs(Vector3.Dot(offset, LocalX)) < (width * 0.5f) && Math.Abs(Vector3.Dot(offset, LocalZ)) < (depth * 0.5f);
            return result;
        }

        /// <summary>
        /// True if supporting some node of o.
        /// </summary>
        public override bool IsSupporting(TwigObject o)
        {
            return SupportingSurfaceDefaultIsSupportingMethod(o);
        }
    }
}
