// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;

#pragma warning disable 1591

namespace Twig
{
    /// <summary>
    /// A cylinder mesh.
    /// </summary>
    public class CylinderMesh
    {
#if OldMesh
        public CylinderMesh(GraphicsDevice device, float radius, float height, int angularResolution)
        {
                float halfHeight = height * 0.5f;
                VertexPositionNormalTexture[] topVerticies = new VertexPositionNormalTexture[angularResolution + 2];
                VertexPositionNormalTexture[] bottomVerticies = new VertexPositionNormalTexture[angularResolution + 2];
                VertexPositionNormalTexture[] circumferenceVerticies = new VertexPositionNormalTexture[2 * angularResolution + 2];
                int index = 0;
                int circumferenceIndex = 0;

                topVerticies[index] = new VertexPositionNormalTexture(new Vector3(0, halfHeight, 0), new Vector3(0, 1, 0),
                                                                        new Vector2(0, 0));
                bottomVerticies[index++] = new VertexPositionNormalTexture(new Vector3(0, -halfHeight, 0), new Vector3(0, -1, 0),
                                                                        new Vector2(0, 0));
                for (int angleIndex = 0; angleIndex < angularResolution + 1; angleIndex++)
                {
                    float theta = angleIndex * (2 * (float)Math.PI / (float)angularResolution);
                    float sinTheta = (float)Math.Sin(theta);
                    float cosTheta = (float)Math.Cos(theta);
                    topVerticies[index] =
                        new VertexPositionNormalTexture(new Vector3(radius * cosTheta, halfHeight, radius * sinTheta),
                                                        new Vector3(0, 1, 0),
                                                        new Vector2(cosTheta, sinTheta));
                    bottomVerticies[index++] =
                        new VertexPositionNormalTexture(new Vector3(-radius * cosTheta, -halfHeight, radius * sinTheta),
                                                        new Vector3(0, -1, 0),
                                                        new Vector2(cosTheta, sinTheta));
                    circumferenceVerticies[circumferenceIndex++] =
                        new VertexPositionNormalTexture(new Vector3(radius * cosTheta, halfHeight, radius * sinTheta),
                                                        new Vector3(cosTheta, 0, sinTheta),
                                                        new Vector2(0, 0));
                    circumferenceVerticies[circumferenceIndex++] =
                        new VertexPositionNormalTexture(new Vector3(radius * cosTheta, -halfHeight, radius * sinTheta),
                                        new Vector3(cosTheta, 0, sinTheta),
                                        new Vector2(0, 0));
#if notdef
            topVerticies[index] =
                new VertexPositionNormalTexture(new Vector3(1, halfHeight, 0),
                                                new Vector3(0, 1, 0), new Vector2(1, 0));
            bottomVerticies[index++] =
                new VertexPositionNormalTexture(new Vector3(-1, -halfHeight, 0),
                                                new Vector3(0, -1, 0), new Vector2(1, 0));
            circumferenceVerticies[circumferenceIndex++] =
                new VertexPositionNormalTexture(new Vector3(radius, halfHeight, 0),
                                                new Vector3(1, 0, 0),
                                                new Vector2(0, 0));
            circumferenceVerticies[circumferenceIndex++] =
                new VertexPositionNormalTexture(new Vector3(radius, -halfHeight, 0),
                                new Vector3(1, 0, 0),
                                new Vector2(0, 0));
#endif

                top = SimpleMesh.TriangleFan(device, topVerticies);
                bottom = SimpleMesh.TriangleFan(device, bottomVerticies);
                circumference = SimpleMesh.TriangleStrip(device, circumferenceVerticies);
            }
        }
        SimpleMesh top, bottom, circumference;

        /// <summary>
        /// Draws mesh.
        /// </summary>
        public void Draw()
        {
            top.Draw();
            bottom.Draw();
            circumference.Draw();
        }
#else
        CylinderMesh(GraphicsDevice device, float radius, float height, int angularResolution)
        {
            MeshBuilder<VertexPositionNormalTexture> b = new MeshBuilder<VertexPositionNormalTexture>(VertexPositionNormalTexture.SizeInBytes);
            this.radius = radius;
            this.halfHeight = height*0.5f;

            b.StartTriangleStrip(TopVertex(0, false), BottomVertex(0, false));
            for (int angleIndex = 1; angleIndex < angularResolution + 1; angleIndex++)
            {
                float theta = angleIndex * (2 * (float)Math.PI / (float)angularResolution);
                b.AddTriangle(TopVertex(theta, false));
                b.AddTriangle(BottomVertex(theta, false));
            }

            b.StartTriangleFan(new VertexPositionNormalTexture(new Vector3(0, halfHeight, 0), Vector3.UnitY, new Vector2(0, 0)),
                TopVertex(0, true));
            for (int angleIndex = 0; angleIndex < angularResolution + 1; angleIndex++)
            {
                float theta = angleIndex * (2 * (float)Math.PI / (float)angularResolution);
                b.AddTriangle(TopVertex(theta, true));
            }

            b.StartTriangleFan(new VertexPositionNormalTexture(new Vector3(0, -halfHeight, 0), -Vector3.UnitY, new Vector2(0, 0)),
    TopVertex(0, true));
            for (int angleIndex = 0; angleIndex < angularResolution + 1; angleIndex++)
            {
                float theta = angleIndex * (2 * (float)Math.PI / (float)angularResolution);
                b.AddTriangle(BottomVertex(-theta, true));
            }

#if notyet
            for (int angleIndex = 0; angleIndex < angularResolution + 1; angleIndex++)
            {
                float theta = angleIndex * (2 * (float)Math.PI / (float)angularResolution);
                float sinTheta = (float)Math.Sin(theta);
                float cosTheta = (float)Math.Cos(theta);
                topVerticies[index] =
                    new VertexPositionNormalTexture(new Vector3(radius * cosTheta, halfHeight, radius * sinTheta),
                                                    new Vector3(0, 1, 0),
                                                    new Vector2(cosTheta, sinTheta));
                bottomVerticies[index++] =
                    new VertexPositionNormalTexture(new Vector3(-radius * cosTheta, -halfHeight, radius * sinTheta),
                                                    new Vector3(0, -1, 0),
                                                    new Vector2(cosTheta, sinTheta));
                circumferenceVerticies[circumferenceIndex++] =
                    new VertexPositionNormalTexture(new Vector3(radius * cosTheta, halfHeight, radius * sinTheta),
                                                    new Vector3(cosTheta, 0, sinTheta),
                                                    new Vector2(0, 0));
                circumferenceVerticies[circumferenceIndex++] =
                    new VertexPositionNormalTexture(new Vector3(radius * cosTheta, -halfHeight, radius * sinTheta),
                                    new Vector3(cosTheta, 0, sinTheta),
                                    new Vector2(0, 0));
            }
#endif
            mesh = new SimpleMesh(device, b);
        }
        SimpleMesh mesh;
        float radius, halfHeight;

        VertexPositionNormalTexture TopVertex(float theta, bool topPiece)
        {
            float sinTheta = (float)Math.Sin(theta);
            float cosTheta = (float)Math.Cos(theta);
            return new VertexPositionNormalTexture(new Vector3(radius * cosTheta, halfHeight, radius * sinTheta),
                topPiece?Vector3.UnitY:new Vector3(cosTheta, 0, sinTheta),
                new Vector2(0, 0));
        }

        VertexPositionNormalTexture BottomVertex(float theta, bool topPiece)
        {
            float sinTheta = (float)Math.Sin(theta);
            float cosTheta = (float)Math.Cos(theta);
            return new VertexPositionNormalTexture(new Vector3(radius * cosTheta, -halfHeight, radius * sinTheta),
                topPiece?(-Vector3.UnitY):new Vector3(cosTheta, 0, sinTheta),
                new Vector2(0, 0));
        }

        struct CylSize
        {
            public CylSize(float l, float r)
            {
                length = l;
                radius = r;
            }

            public float length;
            public float radius;
        }
        static Dictionary<CylSize, CylinderMesh> cylinders = new Dictionary<CylSize, CylinderMesh>();

        public static CylinderMesh Get(GraphicsDevice d, float radius, float height, int angularResolution)
        {
            CylSize s = new CylSize(radius, height);
            if (cylinders.ContainsKey(s))
                return cylinders[s];
            else
            {
                CylinderMesh m = new CylinderMesh(d, radius, height, angularResolution);
                cylinders[s] = m;
                return m;
            }
        }

        /// <summary>
        /// Draws mesh.
        /// </summary>
        public void Draw()
        {
            mesh.Draw();
        }
#endif
    }
}
