// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace Twig
{
    partial class Mammal
    {
        ExploreBehavior exploreBehavior;
        class ExploreBehavior : Behavior<ApproachParameters>
        {
            public override string Name
            {
                get
                {
                    return "Explore";
                }
            }
            Mammal me;
            public ExploreBehavior(Mammal me)
            {
                this.me = me;
            }

            bool inPosition;
            TwigObject target;

            protected override void Update()
            {
                TwigObject focus = me.Attention.Focus;
                if (me.Appraisal(focus).valence >= me.actionThreshold &&
                    (target == null || me.Appraisal(focus).valence > me.Appraisal(target).valence))
                    target = focus;

                if (target != null)
                {
                    Vector3 direction = target.Position;
                    MathUtil.SafeNormalize(ref direction);
                    if (direction==Vector3.Zero
                        || Vector3.Dot(target.Position, direction) + 0.55f < Vector3.Dot(me.Position, direction))
                        inPosition = true;
                    if (inPosition && Vector3.Dot(target.Position, direction) > Vector3.Dot(me.Position, direction)+1)
                        inPosition = false;
                    //inPosition = true;

                    if (!me.crowded && target is Ball && me.Position.LengthSquared() > 10 && !inPosition)
                    {
                        me.WaitingFor("Explore: kick position");
                        MotorOutput.Approach(target, 0.2f, direction*1f);
                    }
                    else if (target is MerryGoRound)
                    {
                        MotorOutput.Approach(target, 1f);
                    }
                    else
                    {
                        me.WaitingFor("Explore: approach target");
                        MotorOutput.Approach(target);
                    }
                }
                Activation = 0.5f*me.Appraisal(target).valence;
                //me.Chart("Activation", Activation, 2);
            }
        }
    }
}
