﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

using Slot = Twig.Mammal.AttentionBuffer.Slot;


namespace Twig
{
    partial class Mammal
    {
        /// <summary>
        /// True if the object is being held by the character.
        /// </summary>
        public Predicate holding;
        Predicate inGraspingPosition;
        Predicate handInGraspingPosition;

        /// <summary>
        /// An action in which the character approaches and grasps the specified object.
        /// </summary>
        public Action Grab(TwigObject o)
        {
            Grasp g;
            Node l, r;
            o.SelectGrasp(this, Grasp.Any, out g, out l, out r);
            if (g == Grasp.None)
                return new FailAction();
            else
                return new GoalAchievementAction(holding, o);
        }


        void InitializeGrasping()
        {
            holding = new Predicate("holding", this, delegate(TwigObject o)
            {
                return IsHolding(o);
            });

            handInGraspingPosition = new Predicate("handInGraspingPosition", this,
                delegate(TwigObject o)
                {
                    return LeftHandInPositionToGrasp(o) || RightHandInPositionToGrasp(o);
                });

            inGraspingPosition = new Predicate("inGraspingPosition", this,
                delegate(Slot s)
                {
                    TwigObject o = Attention.SlotObject(s);
                    return (o==null)?false:(distance[s] < CollisionRadius + o.CollisionRadius + 1f);
                });

            holding.DeclarePreconditions(inGraspingPosition, handInGraspingPosition, facingToward);
            //handInGraspingPosition.DeclarePreconditions(inGraspingPosition, facingToward);

            ApproachParameters graspApproachParameters = new ApproachParameters();
            approachController.Add(new PredicateAchievingBehavior<ApproachParameters>(inGraspingPosition, false,
                delegate(TwigObject o)
                {
                    graspApproachParameters.Approach(o,0);
                    return graspApproachParameters;
                }));
        }

        void GraspControl()
        {
            TwigObject tobject;
            float strength;
            Slot s;

            holding.Goal.FindWinner(out tobject, out strength, out s);
            if (strength > 0 && inGraspingPosition[s])
            {
                // grab it
                tobject.SetupHold(this, Grasp.Any);
            }
        }
    }
}
