// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    /// <summary>
    /// A simple green ground plane.
    /// </summary>
    public class GroundPlane : DrawableGameComponent
    {
        /// <summary>
        /// Makes a simple green ground plane.
        /// </summary>
        public GroundPlane(TwigGame g)
            : base(g)
        {
            twigGame = g;
        }

        TwigGame twigGame;
        GraphicsDevice Device
        {
            get
            {
                return twigGame.Device;
            }
        }

        /// <summary>
        /// Mesh for the ground plane.
        /// </summary>
        protected SimpleMesh groundPlane;
        /// <summary>
        /// Effect to use when rendering ground plane mesh
        /// </summary>
        protected BasicEffect groundPlaneEffect;

        /// <summary>
        /// Initializes ground plane
        /// </summary>
        public override void Initialize()
        {
            BasicEffect groundPlaneEffect = new BasicEffect(Device, null);
            groundPlaneEffect.DiffuseColor = Color.Green.ToVector3();
            groundPlaneEffect.AmbientLightColor = Color.Green.ToVector3() * 0.3f;
            groundPlaneEffect.SpecularColor = Vector3.Zero;

            groundPlaneEffect.LightingEnabled = true;
            groundPlaneEffect.EnableDefaultLighting();

            groundPlaneEffect.View = twigGame.ViewMatrix;
            groundPlaneEffect.Projection = twigGame.ProjectionMatrix;
            groundPlaneEffect.World = Matrix.Identity;
            this.groundPlaneEffect = groundPlaneEffect;

            groundPlane = new SimpleMesh(Device, PrimitiveType.TriangleFan,
                    new VertexPositionNormalTexture[] {
                    new VertexPositionNormalTexture(new Vector3(-9999, 0, -9999), Vector3.Up, new Vector2(0,0)),
                    new VertexPositionNormalTexture(new Vector3(9999, 0, -9999), Vector3.Up, new Vector2(1,0)),
                    new VertexPositionNormalTexture(new Vector3(9999, 0, 9999), Vector3.Up, new Vector2(1,1)),
                    new VertexPositionNormalTexture(new Vector3(-9999, 0, 9999), Vector3.Up, new Vector2(0,1)) },
    2);


            base.Initialize();
        }

        /// <summary>
        /// Renders set (by default, just a ground plane).
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Draw(GameTime gameTime)
        {
            groundPlaneEffect.View = twigGame.ViewMatrix;
            groundPlaneEffect.Projection = twigGame.ProjectionMatrix;
            groundPlaneEffect.CommitChanges();
            groundPlaneEffect.Begin();
            foreach (EffectPass pass in groundPlaneEffect.CurrentTechnique.Passes)
            {
                pass.Begin();
                groundPlane.Draw();
                pass.End();
            }
            groundPlaneEffect.End();
        }
    }
}
