﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Twig
{
    /// <summary>
    /// Stuff for keeping track up updates of predicates and functions
    /// </summary>
    partial class Mammal
    {
        /// <summary>
        /// A procedure that updates a predicate or function.
        /// </summary>
        public delegate void InferenceThunk();

        // This really ought to be an event, but I need to control the order of
        // execution and I don't think events guarantee what order they run
        // their thunks in.
        List<InferenceThunk> updateThunks = new List<InferenceThunk>();

        /// <summary>
        /// Adds an update thunk to the list of updates to perform on each clock cycle.
        /// </summary>
        public void RegisterUpdate(InferenceThunk t)
        {
            updateThunks.Add(t);
        }

        /// <summary>
        /// Updates all functions and predicates
        /// </summary>
        public void UpdateInference()
        {
            for (int i = 0; i < updateThunks.Count; i++)
                updateThunks[i]();
        }
    }
}
