// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace Twig
{
    /// <summary>
    /// A marker is an invisible place marker that can be used to direct characters to specific positions.
    /// </summary>
    public class Marker : TwigObject
    {
        Node node;

        /// <summary>
        /// Place a marker at the desiginated position
        /// </summary>
        public Marker(TwigGame g, string name, Vector3 position)
            : base(g, name, false)
        {
            node = new Node(this, name, position);
            nodes = new Node[] { node };
            collisionElements = new CollisionElement[0];
            Cloaked = true;
        }

        /// <summary>
        /// Position of the marker
        /// </summary>
        public override Vector3 Position
        {
            get { return node.Position; }
        }

        /// <summary>
        /// Markers are massless
        /// </summary>
        public override float Mass
        {
            get { return 0; }
        }

        /// <summary>
        /// Position of the marker
        /// </summary>
        public override Vector3 CenterOfMass
        {
            get { return Position; }
        }

        /// <summary>
        /// Position of the marker
        /// </summary>
        public override Vector3 LookPoint
        {
            get { return Position; }
        }

        /// <summary>
        /// Markers aren't collidable
        /// </summary>
        public override float CollisionRadius
        {
            get { return 0; }
        }

        /// <summary>
        /// Nothing to constraint for markers
        /// </summary>
        public override void Constrain()
        { }
    }
}
