﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    /// <summary>
    /// Extension mehtods for models
    /// </summary>
    public static class ModelExtensions
    {
        /// <summary>
        /// Axis-aligned bounding box of model
        /// Code taken from Microsoft samples
        /// This code has problems - see comments.
        /// </summary>
        public static GeometryPrimitives.AABB BoundingBox(this Model model)
        {
            Vector3[] vertices = null;
            ushort[] indices = null;
            GeometryPrimitives.AABB bb = new GeometryPrimitives.AABB();
            bool first = true;
            foreach (ModelMesh mm in model.Meshes)
            {
                Matrix x = Matrix.Identity;
                ModelBone mb = mm.ParentBone;
                while (mb != null)
                {
                    x = x * mb.Transform;
                    mb = mb.Parent;
                }
                foreach (ModelMeshPart mp in mm.MeshParts)
                {
                    int n = mp.NumVertices;
                    if (vertices == null || n > vertices.Length)
                        // This used to be +128, which causes it to bomb for some models.
                        // need to figure out what this code is really doing and fix it.
                        vertices = new Vector3[n + 1024];
                    int l = mp.PrimitiveCount * 3;
                    if (indices == null || l > indices.Length)
                        // This used to be +128, which causes it to bomb for some models.
                        // need to figure out what this code is really doing and fix it.
                        indices = new ushort[l + 1024];
                    if (n == 0 || l == 0)
                        continue;
                    mm.IndexBuffer.GetData<ushort>(indices, mp.StartIndex, l);
                    mm.VertexBuffer.GetData<Vector3>(mp.StreamOffset, vertices, mp.BaseVertex, n, mp.VertexStride);
                    if (first)
                    {
                        bb.min = Vector3.Transform(vertices[indices[0]], x);
                        bb.max = bb.min;
                        first = false;
                    }
                    for (int i = 0; i != l; ++i)
                    {
                        ushort us = indices[i];
                        Vector3 v = Vector3.Transform(vertices[us], x);
                        Vector3.Max(ref v, ref bb.max, out bb.max);
                        Vector3.Min(ref v, ref bb.min, out bb.min);
                    }
                }
            }
            return bb;
        }
    }
}
