// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    partial class Mammal
    {
        /// <summary>
        /// Total pain level summed over all links (bones).
        /// Decays over time.
        /// </summary>
        public float pain;
        /// <summary>
        /// Link causing maximal pain
        /// </summary>
        Link painLocation;
        /// <summary>
        /// Object colliding with painLocation
        /// </summary>
        TwigObject painCause;

        /// <summary>
        /// Fixed action pattern to move away from source of pain.
        /// Initialized in constructor for Mammal
        /// </summary>
        Behavior<Vector3> painWithdrawalReflex;

        /// <summary>
        /// Fixed action pattern to move away from source of pain.
        /// </summary>
        class PainWithdrawalReflex : Behavior<Vector3>
        {
            Mammal my;
            /// <summary>
            /// Speed with which to move during pain withdrawl
            /// </summary>
            public float painWithdrawlSpeed = 50;

#if DEBUG
            /// <summary>
            /// If true, characters turn red when withdrawing from pain.
            /// </summary>
            [DebugSwitch("Display pain withdrawl as character color")]
            public static bool DisplayPainAsColor = false;
#endif

            public PainWithdrawalReflex(Mammal me)
            {
                my = me;
            }

            Fuse fuse = new Fuse();
            protected override void Update()
            {
                if (my.pain > 0 && my.painCause != null && !fuse.Active)
                {
                    Vector3 withdrawlVector = my.Position - my.painCause.Position;
                    withdrawlVector *= painWithdrawlSpeed / withdrawlVector.Length();
                    MotorOutput = withdrawlVector;
                    Activation = 1;
                    fuse.Fire(0.25f);
                }
                else if (!fuse.Active || my.pain<100)
                {
                    MotorOutput = Vector3.Zero;
                    Activation = 0;
                }

#if DEBUG
                if (DisplayPainAsColor)
                    my.Color = new Color(Activation > 0 ? (byte)255 : (byte)0, 0, mActivation>0?(byte)0:(byte)255);
#endif
            }
        }

        /// <summary>
        /// Checks links to update pain level, link, and cause.
        /// </summary>
        protected void SensePain()
        {
            // Recompute the total pain level
            pain = 0;
            painCause = null;
            painLocation = null;
            float maxPain = 0;

            // HORRIBLE KLUGE
            if (hug || hugging)
                return;

            foreach (Link l in Links)
            {
                float p = l.pain;
                if (p > 0)
                {
                    pain += p;
                    if (p > maxPain)
                    {
                        maxPain = p;
                        painLocation = l;
                        if (l.contactObject!=null)
                            painCause = l.contactObject;
                    }
                }
            }
            Debug.AssertProper(pain, "Pain level is an invalid number");
        }
    }
}
