// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using System.IO;

namespace Twig
{
    /// <summary>
    /// Stores the pose of a character
    /// </summary>
    public class Pose
    {
        /// <summary>
        /// Create a pose object representing the current pose of this character
        /// </summary>
        public Pose(Ragdoll p)
        {
            Node[] nodes = p.Nodes;
            NodePositions = new Vector3[nodes.Length];
            for (int i = 0; i < nodes.Length; i++)
                NodePositions[i] = nodes[i].Position;
        }

        /// <summary>
        /// Positions of the nodes
        /// </summary>
        public Vector3[] NodePositions;

        /// <summary>
        /// Save Pose as an XML file
        /// </summary>
        public void Save(string filename)
        {
            TextWriter o = File.CreateText(filename+".xml");
            o.WriteLine("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
            o.WriteLine("<XnaContent>");
            o.WriteLine("<Asset Type=\"Twig.Pose\">");
            o.WriteLine("   <NodePositions>");
            foreach (Vector3 v in NodePositions)
                o.WriteLine("      <item>{0} {1} {2}</item>", v.X, v.Y, v.Z);
            o.WriteLine("   </NodePositions>");
            o.WriteLine("</Asset>");
            o.WriteLine("</XnaContent>");
            o.Close();
        }
    }
}
