﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;

using Slot = Twig.Mammal.AttentionBuffer.Slot;

namespace Twig
{
    /// <summary>
    /// A predicate that's activated by goal assertions on a predicate
    /// </summary>
    /// <typeparam name="T"></typeparam>
    class PredicateAchievingBehavior<T> : Behavior<T> where T : new()
    {
        Predicate predicate;
        bool persistent;

        /// <summary>
        /// Computes a motor vector to achieve the predicate on object o.
        /// 
        /// WARNING: o may be null!
        /// </summary>
        public delegate T MotorController(TwigObject o);

        MotorController controller;

        /// <summary>
        /// Creates a behavior that tries to achieve a predicate when it is asserted as a goal
        /// 
        /// WARNING: controller must tolerate a null input
        /// </summary>
        /// <param name="p">Predicate that drives the behavior; when this predicate is made a goal, it will activate the behavior</param>
        /// <param name="persistent">Whether to keep the behavior active once the predicate is achieved</param>
        /// <param name="c">Controller to achieve the predicate</param>
        public PredicateAchievingBehavior(Predicate p, bool persistent, MotorController c)
        {
            predicate = p;
            controller = c;
            this.persistent = persistent;
        }

        protected override void Update()
        {
            TwigObject target;
            float strength;
            Slot s;

            predicate.Goal.FindWinner(out target, out strength, out s);
            if (target==null || (!persistent && predicate[s]))
                // Don't fire if no target or if predicate is already achieved, unless persistent
                strength = 0;
            mActivation = strength;
            mOutput = controller(strength>0?target:null);
        }
    }
}
