﻿// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    class ProtestSign : Paper
    {
        Node topMidpoint;
        Node bottomMidpoint;
        Node handleBottom;
        LinkWithMidpoint topLink;
        LinkWithMidpoint bottomLink;
        LinkWithMidpoint handle;

        public ProtestSign(TwigGame g, string name, string modelName, Vector3 position)
            : base(g, name, modelName, position)
        {
            topMidpoint = new Node(this, name + "-topMidpoint");
            bottomMidpoint = new Node(this, name + "-bottomMidpoint");
            handleBottom = new Node(this, name + "-handleBottom");
            topLink = new LinkWithMidpoint(name + "-topLink", this, nodes[4], topMidpoint, nodes[5], depth, 0);
            bottomLink = new LinkWithMidpoint(name + "-topLink", this, nodes[0], bottomMidpoint, nodes[1], depth, 0);
            handle = new LinkWithMidpoint(name + "-handle", this, topMidpoint, bottomMidpoint, handleBottom, 2 * width, 0.02f);
            AddNodes(topMidpoint, bottomMidpoint, handleBottom);
            AddCollisionElements(handle.CollisionElement);
        }

        public override void Update(GameTime gameTime)
        {
            //topMidpoint.Position = new Vector3(0, 3, 0);
            topMidpoint.Update();
            bottomMidpoint.Update();
            handleBottom.Update();
            base.Update(gameTime);
        }

        public override void Constrain()
        {
            for (int i = 0; i < 1; i++)
            {
                topLink.Constrain();
                bottomLink.Constrain();
                handle.Constrain();
                base.Constrain();
            }
        }

        public override void Draw(GameTime gameTime)
        {
            TwigGame.BasicEffect.DiffuseColor = Color.Brown.ToVector3();
            TwigGame.BasicEffect.Begin();
            foreach (EffectPass p in TwigGame.BasicEffect.CurrentTechnique.Passes)
            {
                p.Begin();
                handle.Draw();
                p.End();
            }
            TwigGame.BasicEffect.End();
            base.Draw(gameTime);
        }

        /// <summary>
        /// Tells paper to position itself for holding by holder; also tells holder to grab paper.
        /// </summary>
        public override void SetupHoldForUse(Mammal holder, Grasp grasp)
        {
            SetupHold(holder, grasp);
            Place(holder.Atlas["torso"], new Vector3(0.4f, 3f, 0.4f), "front", Vector3.UnitZ);
        }

        /// <summary>
        /// Tells paper to position itself for holding by holder; also tells holder to grab paper.
        /// </summary>
        public override void SetupHold(Mammal holder, Grasp grasp)
        {
            holder.Hold(handleBottom, handleBottom, this, grasp);
            Place(null, Vector3.Zero, null, null);
        }

        /// <summary>
        /// Select hands and nodes for grasping.  Will choose handle-bottom.
        /// </summary>
        public override void SelectGrasp(Mammal grasper, Grasp graspRequest,
                                         out Grasp selectedGrasp, out Node leftTarget, out Node rightTarget)
        {
            grasper.SelectGrasp(handleBottom, handleBottom, this, graspRequest,
                                out selectedGrasp, out leftTarget, out rightTarget);
        }

    }
}