// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

namespace Twig
{
    //
    // The Security/Anxiety system estimates the degree to
    // which the agent's control policies and coping mechanisms
    // are sufficient for the immediate situation.  Insecurity
    // (anxiety) triggers coping mechanism (increased vigilance,
    // attachment, caregiving, etc.)
    //
    partial class Mammal
    {
        float m_security=1f;
        /// <summary>
        /// Basal level of security.
        /// This is the security level of the creature in the absence of a care taker or threats.
        /// </summary>
        public float baseLineSecurity;
        /// <summary>
        /// Creature's subjective level of security, usually in the range 0 (mortal peril) to 1 (very secure).
        /// </summary>
        public float Security
        {
            get
            {
                return m_security;
            }
        }

        /// <summary>
        /// Anxiety level.  Defined as 1-Security.
        /// </summary>
        public float Anxiety
        {
            get
            {
                return 1 - Security;
            }
        }

        void UpdateSecurity()
        {
            float instantaneous = baseLineSecurity + AttachmentSecurity;
            float weight = 0.002f;

            m_security = Math.Min(1f, Math.Max(0f, (1f - weight) * m_security + weight * instantaneous));
        }
    }
}
